@@ -35,6 +35,77 @@ fn maybe_duration<'d, D: serde::Deserializer<'d>>(d: D)
 	d.deserialize_str(V).map(Some)
 }
 
+fn write_elapsed(
+	mut buf: impl std::io::Write,
+	duration: std::time::Duration)
+-> std::io::Result<()>
+{
+	write!(buf, "Last output ")?;
+	write_duration(&mut buf, duration)?;
+	write!(&mut buf, " ago.")
+}
+
+fn write_duration(
+	mut buf: impl std::io::Write,
+	duration: std::time::Duration)
+-> std::io::Result<()>
+{
+	let mut sec = duration.as_secs();
+	if duration.as_secs() >= 24*3600 {
+		let days = sec / 3600 / 24;
+		sec -= days * 24*3600;
+		write!(buf, "{}d ", days)?;
+	}
+	if duration.as_secs() >= 3600 {
+		let hours = sec / 3600;
+		sec -= hours * 3600;
+		write!(buf, "{:2}h ", hours)?;
+	}
+	if duration.as_secs() >= 60 {
+		let min = sec / 60;
+		sec -= min * 60;
+		write!(buf, "{:2}min ", min)?;
+	}
+	write!(buf, "{:2}s", sec)
+}
+
+#[test]
+fn test_format_duration() {
+	let mut buf = Vec::new();
+
+	buf.clear();
+	write_duration(&mut buf, std::time::Duration::from_secs(0)).unwrap();
+	assert_eq!(std::str::from_utf8(&buf).unwrap(), " 0s");
+
+	buf.clear();
+	write_duration(&mut buf, std::time::Duration::from_secs(1)).unwrap();
+	assert_eq!(std::str::from_utf8(&buf).unwrap(), " 1s");
+
+	buf.clear();
+	write_duration(&mut buf, std::time::Duration::from_secs(59)).unwrap();
+	assert_eq!(std::str::from_utf8(&buf).unwrap(), "59s");
+
+	buf.clear();
+	write_duration(&mut buf, std::time::Duration::from_secs(60)).unwrap();
+	assert_eq!(std::str::from_utf8(&buf).unwrap(), " 1min  0s");
+
+	buf.clear();
+	write_duration(&mut buf, std::time::Duration::from_secs(61)).unwrap();
+	assert_eq!(std::str::from_utf8(&buf).unwrap(), " 1min  1s");
+
+	buf.clear();
+	write_duration(&mut buf, std::time::Duration::from_secs(119)).unwrap();
+	assert_eq!(std::str::from_utf8(&buf).unwrap(), " 1min 59s");
+
+	buf.clear();
+	write_duration(&mut buf, std::time::Duration::from_secs(3662)).unwrap();
+	assert_eq!(std::str::from_utf8(&buf).unwrap(), " 1h  1min  2s");
+
+	buf.clear();
+	write_duration(&mut buf, std::time::Duration::from_secs(76*3600 + 18)).unwrap();
+	assert_eq!(std::str::from_utf8(&buf).unwrap(), "3d  4h  0min 18s");
+}
+
 #[derive(Debug,serde_derive::Deserialize)]
 #[serde(deny_unknown_fields)]
 #[serde(rename_all="kebab-case")]
@@ -130,7 +201,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
 		if !new_line {
 			buf.extend("…\n".as_bytes())
 		}
-		write!(&mut buf, "Last output {:.0?} ago.", last_read.elapsed())?;
+		write_elapsed(&mut buf, last_read.elapsed())?;
 		stdout.write_all(&buf)?;
 		stdout.flush()?;
 		buf.clear();
@@ -168,9 +239,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
 
 			buf.extend(ANSII_MOVE_COL_1);
 			buf.extend(ANSII_LINE_CLEAR_END);
-			write!(&mut buf,
-				"Last output {:.0?} ago.",
-				last_read.elapsed())?;
+			write_elapsed(&mut buf, last_read.elapsed())?;
 			stdout.write_all(&buf)?;
 			stdout.flush()?;
 			buf.clear();
